%% State-Space Averaging: Open-Loop Transfer Functions of a Converter

%% Load model
load_system('plBuckSweep');
open_system('plBuckSweep/Circuit');

%% Determine switch order
names = plecsedit('get', 'plBuckSweep/Circuit', 'StateSpaceOrder');
names.Switches

%% Retrieve state-space matrices
plecsedit('set', 'plBuckSweep/Circuit', 'SwitchVector', [1 0]);
t1 = plecsedit('get', 'plBuckSweep/Circuit', 'Topology')
plecsedit('set', 'plBuckSweep/Circuit', 'SwitchVector', [0 1]);
t2 = plecsedit('get', 'plBuckSweep/Circuit', 'Topology')

%% Calculate averaged state-space model
m = 15/28;
A = t1.A*m + t2.A*(1-m);
B = t1.B*m + t2.B*(1-m);
C = t1.C*m + t2.C*(1-m);
D = t1.D*m + t2.D*(1-m);

%% Determine input and output indices
names.Inputs
names.Outputs

%% Calculate transfer function from input 2 to output 1
% |bode(ss(A,B(:,2),C(1,:),D(1,2)), {2*pi*10, 2*pi*50000})|
F = logspace(1, 5, 201);
Z = zeros(size(F));
I = eye(size(A));
for idx = 1:length(F),
  Z(idx) = C(1,:) * ((2i*pi*F(idx)*I - A)\B(:,2)) + D(1,2);
end

%% Create bode plot
figure('Color', [1 1 1]);
subplot(211);
semilogx(F, 20*log10(abs(Z)));
grid on
title('Output Impedance');
ylabel('Amplitude / dB\Omega');
subplot(212);
semilogx(F, unwrap(angle(Z))/pi*180);
grid on
ylabel('Phase / \circ');
xlabel('Frequency / Hz');

%% Calculate small-signal model for variation of duty cycle
u = [28; 0];
B_co = (-(t1.A-t2.A)*(A\B)+(t1.B-t2.B))*u;
D_co = (-(t1.C-t2.C)*(A\B)+(t1.D-t2.D))*u;

%% Calculate control-to-output transfer function
% |bode(ss(A,B_co,C(1,:),D_co(1)), {2*pi*10, 2*pi*50000})|
G_co = zeros(size(F));
for idx = 1:length(F),
  G_co(idx) = C(1,:) * ((2i*pi*F(idx)*I - A)\B_co) + D_co(1);
end

%% Create bode plot
figure('Color', [1 1 1]);
subplot(211);
semilogx(F, 20*log10(abs(G_co)));
grid on
title('Control to Output Transfer Function');
ylabel('Amplitude / dB');
subplot(212);
semilogx(F, unwrap(angle(G_co))/pi*180);
grid on
ylabel('Phase / \circ');
xlabel('Frequency / Hz');
